{-|
Module      : Gargantext.Core.Worker.Jobs.Types
Description : Worker job definitions
Copyright   : (c) CNRS, 2024
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}


module Gargantext.Core.Worker.Jobs.Types where


import Data.Aeson ((.:), (.=), object, withObject)
import Data.Aeson.Types (prependFailure, typeMismatch)
import Gargantext.API.Admin.EnvTypes ( GargJob )
import Gargantext.Prelude


data Job =
    Ping
  | ForgotPassword { _fp_email :: Text }
  | GargJob { _gj_garg_job :: GargJob }
  deriving (Show, Eq)
instance FromJSON Job where
  parseJSON = withObject "Job" $ \o -> do
    type_ <- o .: "type"
    case type_ of
      "Ping" -> return Ping
      "ForgotPassword" -> do
        _fp_email <- o .: "email"
        return $ ForgotPassword { _fp_email }
      "GargJob" -> do
        _gj_garg_job <- o .: "garg_job"
        return $ GargJob { _gj_garg_job }
      s -> prependFailure "parsing job type failed, " (typeMismatch "type" s)
instance ToJSON Job where
  toJSON Ping = object [ ("type" .= ("Ping" :: Text)) ]
  toJSON (ForgotPassword { _fp_email }) = object [ ("type" .= ("ForgotPassword" :: Text))
                                                 , ("email" .= _fp_email) ]
  toJSON (GargJob { _gj_garg_job }) = object [ ("type" .= ("GargJob" :: Text))
                                             , ("garg_job" .= _gj_garg_job) ]
